/**@file
 *
 *@brief	黻
 *@date		Sat,30 Dec,2006 - Thu,11 Jan,2007
 *@date		Mon,21 Apr,2008
 *@date		Fri,27 Jun,2008
 *@date		Sat,24 Aug,2013 - Mon,26 Aug,2013, Sat,31 Aug,2013
 *@date		Tue,03 Sep,2013
 *@author	Copyright(C)2006-2013 G-HAL, All rights reserved.
 *
 *@note
 *@verbatim
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holders nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *@endverbatim
 */
#ifndef calctime_h_included_
#define calctime_h_included_

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif

#if !defined(__STDC_LIMIT_MACROS)
# define __STDC_LIMIT_MACROS
#endif
#if !defined(__STDC_CONSTANT_MACROS)
# define __STDC_CONSTANT_MACROS
#endif
#if defined(HAVE_STDINT_H)
# include <stdint.h>
#endif

#if defined(HAVE_ASSERT_H)
# include <assert.h>
#endif
#if defined(HAVE_MATH_H)
# include <math.h>
#endif
#if defined(HAVE_SYS_TIME_H)
# include <sys/time.h>
#endif


#define TIME_UNIT_MS INT32_C(1000)
#define TIME_UNIT_US INT32_C(1000000)
#define TIME_UNIT_NS INT32_C(1000000000)


/** ʲû
 *@param[in,out]	tv					
 *
 *@author	G-HAL
 */
inline static void NormalizeTime_atAdd(struct timeval* const tv)
{
	assert (NULL != tv);
	const int_fast32_t adj = (int_fast32_t)(floor((double)(tv->tv_usec) / TIME_UNIT_US));
	if (0 < adj) {
		tv->tv_sec  += adj;
		tv->tv_usec -= (adj * TIME_UNIT_US);
	}
	return;
}

/** ʸ
 *@param[in,out]	tv					
 *
 *@author	G-HAL
 */
inline static void NormalizeTime_atSub(struct timeval* const tv)
{
	assert (NULL != tv);
	if (tv->tv_usec < 0) {
		const int_fast32_t adj = - (int_fast32_t)(floor((double)(tv->tv_usec) / TIME_UNIT_US));
		tv->tv_sec  -= adj;
		tv->tv_usec += (adj * TIME_UNIT_US);
	}
	return;
}

/** βû
 *@param[in,out]	src1				浪ӷ̡
 *@param[in]		src2				­
 *@return								ʷ̡
 *
 *@author	G-HAL
 */
inline static void AddTime(struct timeval* const src1, const struct timeval* const src2)
{
	assert (NULL != src1);
	assert (NULL != src2);
	src1->tv_sec  += src2->tv_sec;
	src1->tv_usec += src2->tv_usec;
	NormalizeTime_atAdd( src1 );
	return;
}

/** ΰ
 *@param[in,out]	src1				浪ӷ̡
 *@param[in]		src2				
 *@return								ʷ̡
 *
 *@author	G-HAL
 */
inline static void SubTime(struct timeval* const src1, const struct timeval* const src2)
{
	assert (NULL != src1);
	assert (NULL != src2);
	src1->tv_sec  -= src2->tv_sec;
	src1->tv_usec -= src2->tv_usec;
	NormalizeTime_atSub( src1 );
	return;
}


#endif /* calctime_h_included_ */
/* [ End of File ] */
