/**@file
 *@brief	anthy-agentץȥ  cannaץȥ ̿
 *@date		Fri,23 Aug,2013 - Tue,01 Oct,2013
 *@author	Copyright(C)2013 G-HAL, All rights reserved.
 *
 *@note
 *@verbatim
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holders nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *@endverbatim
 */

#if defined(HAVE_CONFIG_H)
# include "config.h"
# include "revision_header.h"
#endif

#if defined(HAVE_STDLIB_H)
# include <stdlib.h>
#endif
#if defined(HAVE_STDIO_H)
# include <stdio.h>
#endif
#if defined(HAVE_STDBOOL_H)
# include <stdbool.h>
#endif

#if defined(HAVE_ASSERT_H)
# include <assert.h>
#endif

#if defined(HAVE_ERRNO_H)
# include <errno.h>
#endif
#if defined(HAVE_STRING_H)
# include <string.h>
#endif
#if defined(HAVE_SIGNAL_H)
# include <signal.h>
#endif
#if defined(HAVE_SYS_TIME_H)
# include <sys/time.h>
#endif
#if defined(HAVE_SYS_TYPES_H)
# include <sys/types.h>
#endif
#if defined(HAVE_SYS_STAT_H)
# include <sys/stat.h>
#endif

#if defined(HAVE_LOCALE_H)
# include <locale.h>
#endif

#include "intl.h"
#include "logger.h"
#include "args.h"
#include "calctime.h"
#include "parseline.h"



/** оݤΥʥޥ
 *@param[out]		sig					оݤΥʥ
 *
 *@note
 *		δؿ static ʤΤա
 *		const ʴؿʤ
 *@attention
 *		SIGINT(2) å ^C ʤʤ롣
 *		BSDϤξ硢SIGTSTP(18) νå ^Z ʤʤ롣
 *		LinuxϤξ硢SIGCHLD(20) ޥ ^Z ʤʤ롣
 *		LinuxϤξ硢SIGRTMIN+1 ޥ pthread_cancel() Ǥʤʤ롣
 *		Solarisξ硢SIGTSTP(24) ޥ ^Z ʤʤ롣
 *		Solarisξ硢SIGCANCEL ޥ pthread_cancel() Ǥʤʤ롣
 *
 *@author	G-HAL
 *@date		Sat,24 Aug,2013
 */
static void get_supported_sigmask(sigset_t* const sig)
{
	assert (NULL != sig);

	sigemptyset(sig);
  #if 0
	sigaddset(sig, SIGHUP);			/* ġĲ̵ */
	sigaddset(sig, SIGINT);			/* ͥپ夲 */
	sigaddset(sig, SIGQUIT);		/*  */
	sigaddset(sig, SIGKILL);		/* ͥپ夲 */
  #endif
	sigaddset(sig, SIGPIPE);		/*  */
  #if 0
	sigaddset(sig, SIGALRM);		/*  */
	sigaddset(sig, SIGTERM);		/*  */
	sigaddset(sig, SIGVTALRM);		/* ͥپ夲 */
	sigaddset(sig, SIGPROF);		/* ͥپ夲 */
	sigaddset(sig, SIGUSR1);		/* ƥåɤǤν */
	sigaddset(sig, SIGUSR2);		/* ƥåɤǤν */
  #endif
	return;
}



/** SIGPIPE(13) դ
 *@param			var					
 *
 *@note
 *		δؿ static ʤΤա
 *
 *@author	G-HAL
 *@date		Sat,24 Aug,2013
 */
static RETSIGTYPE signal_pipe(_unused int var)
{
  #if defined(SIGNAL_UNSAFE)
	LOGGER_INFO(l, "SIGPIPE(%d): thread = %"PRIpthread_t"", var, pthread_self());
  #else
  #endif
	return;
}



/** main()
 *
 *@author	G-HAL
 *@date		Fri,23 Aug,2013 - Tue,03 Sep,2013
 *@date		Tue,10 Sep,2013
 *@date		Wed,11 Sep,2013
 *@date		Fri,20 Sep,2013
 *@date		Sun,22 Sep,2013
 *@date		Mon,30 Sep,2013
 */
int main(_unused int argc, _unused char* const argv[], _unused char* const env[])
{
	assert (NULL != argv);
	assert (NULL != env);


	int ret = 0;

	/* 롿gettext  */
	{
		_unused_ const char* const old_locale = setlocale(LC_ALL, "");
		_unused_ const char* const new_locale = setlocale(LC_ALL, NULL);
	  #if defined(ENABLE_NLS)
		const char* ret_btd = bindtextdomain(PACKAGE, LOCALEDIR);
		if (NULL == ret_btd) {
			fprintf(stderr,"Out of memory at bindtextdomain().\n");
			abort();
		}
		const char* ret_td = textdomain(PACKAGE);
		if (NULL == ret_td) {
			fprintf(stderr,"Out of memory at textdomain().\n");
			abort();
		}
	  #endif
	}


	/*  */
	{
		init_logger();
		{
			const bool ret_init_args = init_args();
			if (!ret_init_args) {
				ret = -127;
				goto ERR_FIN_LOGGER;
			}
		}

		{
			const bool ret_parse_args = parse_args(argc, argv);
			if (!ret_parse_args) {
				ret = -126;
				goto ERR_FIN_ARGS;
			}
		}

		{	/* ʥޥ
			 * POSIXͤǤϡʹߤ줿ҥåɤϡ
			 * ΥޥѤȦ
			 */
			sigset_t sig;
			get_supported_sigmask(&sig);
			sigprocmask(SIG_BLOCK, &sig, NULL);
			/* pthread_sigmask(SIG_BLOCK, &sig, NULL); */
		}
		{
			struct sigaction sa;
			memset(&sa, 0, sizeof(sa));
			sigemptyset(&sa.sa_mask);
			sa.sa_flags = SA_RESTART;

			sa.sa_handler = signal_pipe;
			const int ret_sigaction = sigaction( SIGPIPE, &sa, NULL );
			if (-1 == ret_sigaction) {
				LOGGER_ERR(l,N_("Can not sigaction(SIGPIPE). %1$s"), strerror(errno));
				ret = -125;
				goto ERR_FIN_ARGS;
			}
		}
	}


	/*  */
  #if defined(BENCHMARK)
	struct timeval start_tod;
	{
		const int ret_gettimeofday = gettimeofday(&start_tod, NULL);
		if (-1 == ret_gettimeofday) {
			LOGGER_ERR(l,N_("Can not calculate time. %1$s"), strerror(errno));
		}
	}
  #endif


	const char* socket_name = (PF_UNIX == args.server.domain) ? args.server.socket_name : args.server.server_name;
	if (PF_UNIX == args.server.domain) {
		struct stat sb;
		const int ret_stat = stat(socket_name, &sb);
		if (-1 == ret_stat) {
			socket_name = args.server.socket_name_fallback;
		}
	}
	struct agent_parser_transaction_t* const apt = init_parser(args.server.domain, socket_name, args.server.service_name, args.user.group, args.user.name, args.cannaprotocol.locale);
	if (NULL == apt) {
		LOGGER_ERR(l,N_("Can not initialize."));
		ret = -124;
		goto ERR_FIN_ARGS;
	}


	/* 롼 */
	{
		char* const buf = malloc((size_t)INPUT_BUFSIZE+1);
		if (NULL == buf) {
			LOGGER_CRIT(l,N_("Out of memory at %1$s"), "main()");
			ret = -123;
			goto ERR_FIN_PARSER;
		}
		while (1) {
			const char* const ptr = fgets(buf, INPUT_BUFSIZE, stdin);
			if (NULL == ptr) {
				break;
			}
			parseline(apt, ptr);
		}
		free(buf);
	}


	/* λ */
ERR_FIN_PARSER:
	{
		const bool ret_fp = fin_parser(apt);
		if (!ret_fp) {
			LOGGER_ERR(l,N_("Can not finalization."));
			if (0 == ret) {
				ret = -122;
			}
		}
	}


  #if defined(BENCHMARK)
	{
		struct timeval done_tod;
		const int ret_gettimeofday = gettimeofday(&done_tod, NULL);
		if (-1 == ret_gettimeofday) {
			LOGGER_ERR(l,N_("Can not calculate time. %1$s"), strerror(errno));
		} else {
			SubTime(&done_tod, &start_tod);
			LOGGER_INFO(l, "Total time:%11"PRId64".%06"PRId64".",
					(int_fast64_t)(done_tod.tv_sec),
					(int_fast64_t)(done_tod.tv_usec)
			);
		}
	}
  #endif

ERR_FIN_ARGS:
	fin_args();
ERR_FIN_LOGGER:
	fin_logger();

	fclose(stdin);
	fclose(stdout);
	fclose(stderr);
  #if 0 /* defined(__linux__) */
	fcloseall();
  #endif
	return ret;
}

/* [ End of File ] */
