/**@file
 *@brief	anthy-agentץȥʹؿϢ
 *@date		Fri,23 Aug,2013 - Mon,30 Sep,2013
 *@author	Copyright(C)2013 G-HAL, All rights reserved.
 *
 *@note
 *@verbatim
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holders nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *@endverbatim
 */

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif

#if defined(HAVE_STDLIB_H)
# include <stdlib.h>
#endif
#if defined(HAVE_STDIO_H)
# include <stdio.h>
#endif
#if defined(HAVE_STDBOOL_H)
# include <stdbool.h>
#endif

#if defined(HAVE_ASSERT_H)
# include <assert.h>
#endif

#if defined(HAVE_STRING_H)
# include <string.h>
#endif

#if defined(HAVE_SYS_UN_H)
# include <sys/un.h>
#endif
#if defined(HAVE_NETINET_IN_H)
# include <netinet/in.h>
#endif

#include "intl.h"
#include "logger.h"
#include "iconv_wrapper.h"
#include "parseline_common.h"
#include "parseline_rk.h"
#include "parseline.h"
#include "cannaprotocol.h"



/** ƥȤ
 *@param[in,out]	apt					̿Ѿ
 *@param			id					ֹ
 *@retval			NULL				顼
 *@return								̿Ѿ
 *
 *@author	G-HAL
 *@date		Wed,28 Aug,2013 - Fri,30 Aug,2013
 *@date		Thu,05 Sep,2013
 *@date		Sun,08 Sep,2013
 *@date		Mon,09 Sep,2013
 *@date		Wed,11 Sep,2013
 *@date		Sat,14 Sep,2013, Mon,16 Sep,2013
 */
struct agent_parser_context_transaction_t* init_parser_context(struct agent_parser_transaction_t* const apt, int id)
{
	assert (NULL != apt);

	/* ɥåȤ߽ */
	{
		const bool ret_ii = init_iconv();
		if (!ret_ii) {
			return NULL;
		}
	}

	/* Τν */
	{
		const bool ret_ipr = init_parser_rk();
		if (!ret_ipr) {
			return NULL;
		}
	}
	struct agent_parser_context_transaction_t* const apct = malloc(sizeof(*apct));
	if (NULL == apct) {
		LOGGER_CRIT(l,N_("Out of memory at %1$s"), "init_parser_context()");
		return NULL;
	}
	{
		apct->apt = apt;
		apct->next = apt->list_apct;
		apct->id = id;
		apct->input_state = PARSELINE_INPUT_STATE_INIT;
		apct->rk_map_table = &(rk_map_tables[0]);
		apct->toggle_key = PARSELINE_TOGGLE_KEY;
		apct->rk_map_direct_mode = false;
		apct->break_into_roman = false;
		apct->preedit_mode = false;
		apct->input.pending_str[0] = '\0';
		apct->input.len = 0;
		apct->input.cursor_ptr = 0;
		apct->commit.str[0] = '\0';
	}

	/* ³ */
	apct->cpct = init_canna_context(apt->cpt);
	if (NULL == apct->cpct) {
		LOGGER_ERR(l,N_("Can not initialize."));
		free(apct);
		return NULL;
	}

	apt->list_apct = apct;
	return apct;
}


/** ФȤ̿򳫻
 *@param			domain				̿ΥɥᥤPF_UNIX  PF_INET Τб
 *@param[in]		name				PF_UNIX: å̾PF_INET: ̾
 *@param[in]		serv				PF_INET: ФΥӥ̾
 *@param[in]		group				롼̾NULL: ꤷʤ
 *@param[in]		user				桼̾
 *@param[in]		locale				̾
 *@retval			NULL				顼
 *@return								̿Ѿ
 *
 *@author	G-HAL
 *@date		Wed,28 Aug,2013 - Fri,30 Aug,2013
 *@date		Thu,05 Sep,2013
 *@date		Sun,08 Sep,2013
 *@date		Mon,09 Sep,2013
 *@date		Sun,22 Sep,2013
 *@date		Mon,30 Sep,2013
 */
struct agent_parser_transaction_t* init_parser(int domain, const char name[], const char serv[], const char group[], const char user[], const char locale[])
{
	assert (NULL != name);
	assert (NULL != serv);
	/* assert (NULL != group); */
	assert (NULL != user);
	/* assert (NULL != locale); */


	/* Τν */
	struct agent_parser_transaction_t* const apt = malloc(sizeof(*apt));
	if (NULL == apt) {
		LOGGER_CRIT(l,N_("Out of memory at %1$s"), "init_parser()");
		return NULL;
	}

	apt->cpt = init_canna(domain, name, serv, group, user, locale);
	if (NULL == apt->cpt) {
		free(apt);
		return NULL;
	}

	apt->list_apct = NULL;
	const struct agent_parser_context_transaction_t* ptr = init_parser_context(apt, 0);
	if (NULL == ptr) {
		free(apt);
		return NULL;
	}

	return apt;
}



/** ƥȤλ
 *@param[in,out]	apct				̿Ѿ
 *@retval			true				
 *@retval			false				
 *
 *@author	G-HAL
 *@date		Wed,28 Aug,2013
 *@date		Mon,09 Sep,2013
 */
bool fin_parser_context(struct agent_parser_context_transaction_t* const apct)
{
	assert (NULL != apct);

	bool ret = false;

	/* ƥȽλ */
	if (!fin_canna_context(apct->cpct)) {
		LOGGER_ERR(l,N_("Can not finalization."));
		ret = false;
	}

	/* ƥȤտ魯 */
	bool hit = false;
	struct agent_parser_context_transaction_t** prev_ptr = &(apct->apt->list_apct);
	for (struct agent_parser_context_transaction_t* ptr = apct->apt->list_apct; NULL != ptr; ptr = ptr->next, prev_ptr = &(ptr->next)) {
		if (apct == ptr) {
			hit = true;
			(*prev_ptr) = ptr->next;
			break;
		}
	}
	if (hit) {
		ret = true;
		free(apct);
	} else {
		ret = false;
		LOGGER_EMERG(l,N_("Data list was broken at %1$s"), "fin_parser_context()");
	}
	return ret;
}


/** ФȤ̿λ
 *@param[in,out]	apt					̿Ѿ
 *@retval			true				
 *@retval			false				
 *
 *@author	G-HAL
 *@date		Wed,28 Aug,2013
 *@date		Mon,09 Sep,2013
 *@date		Wed,11 Sep,2013
 *@date		Mon,16 Sep,2013
 */
bool fin_parser(struct agent_parser_transaction_t* const apt)
{
	assert (NULL != apt);

	bool ret = true;

	/* ƥȲ */
	for (struct agent_parser_context_transaction_t* ptr = apt->list_apct; NULL != ptr;) {
		struct agent_parser_context_transaction_t* const next_ptr = ptr->next;
		const bool ret_fin_pc = fin_parser_context(ptr);
		if (!ret_fin_pc) {
			ret = false;
			break;
		}
		ptr = next_ptr;
	}

	/* ̿λ */
	if (!fin_canna(apt->cpt)) {
		LOGGER_ERR(l,N_("Can not finalization."));
		ret = false;
	}

	/* Τθ */
	free(apt);
	{
		const bool ret_fpr = fin_parser_rk();
		if (!ret_fpr) {
			ret = false;
		}
	}

	/* ɥåȤ߸ */
	{
		const bool ret_fi = fin_iconv();
		if (!ret_fi) {
			ret = false;
		}
	}

	return ret;
}

/* [ End of File ] */
