/**@file
 *@brief	Type Punned Cast
 *@date		Thu,06 Dec,2012 - Fri,14 Dec,2012, Tue,18 Dec,2012
 *@date		Sat,04 May,2013
 *@date		Sun,13 Nov,2016
 *@author	Copyright(C)2012-2016 G-HAL, All rights reserved.
 *
 *@note
 *@verbatim
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

* Neither the name of the copyright holders nor the names of its contributors
may be used to endorse or promote products derived from this software
without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *@endverbatim
 */
#ifndef type_punned_cast_h_included_
#define type_punned_cast_h_included_

#if defined(HAVE_CONFIG_H)
# include "config.h"
#endif

#if !defined(__STDC_LIMIT_MACROS)
# define __STDC_LIMIT_MACROS
#endif
#if !defined(__STDC_CONSTANT_MACROS)
# define __STDC_CONSTANT_MACROS
#endif
#if defined(HAVE_STDINT_H)
# include <stdint.h>
#endif



#ifdef __cplusplus
extern "C" {
#endif



/** Type Punned Cast
 */
union type_punned_cast_t {
	uint8_t u8;
	uint16_t u16;
	uint32_t u32;
	uint64_t u64;
	int8_t s8;
	int16_t s16;
	int32_t s32;
	int64_t s64;
};


/** Type Punned Cast
 */
union type_punned_pointer_cast_t {
	void* vp;
	uint8_t* u8p;
	uint16_t* u16p;
	uint32_t* u32p;
	uint64_t* u64p;
	int8_t* s8p;
	int16_t* s16p;
	int32_t* s32p;
	int64_t* s64p;
};


/** Ѵ
 *@param[in,out]	ptr					ѴγǼ
 *@param			u8					Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline void type_punned_cast_u8_to(void* const ptr, uint8_t u8)
{
	union type_punned_cast_t* const tmp = ptr;
	tmp->u8 = u8;
	return;
}

/** Ѵ
 *@param[in,out]	ptr					ѴγǼ
 *@param			u16					Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline void type_punned_cast_u16_to(void* const ptr, uint16_t u16)
{
	union type_punned_cast_t* const tmp = ptr;
	tmp->u16 = u16;
	return;
}

/** Ѵ
 *@param[in,out]	ptr					ѴγǼ
 *@param			u32					Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline void type_punned_cast_u32_to(void* const ptr, uint32_t u32)
{
	union type_punned_cast_t* const tmp = ptr;
	tmp->u32 = u32;
	return;
}

/** Ѵ
 *@param[in,out]	ptr					ѴγǼ
 *@param			u64					Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline void type_punned_cast_u64_to(void* const ptr, uint64_t u64)
{
	union type_punned_cast_t* const tmp = ptr;
	tmp->u64 = u64;
	return;
}

/** Ѵ
 *@param[in,out]	ptr					ѴγǼ
 *@param			s8					Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline void type_punned_cast_s8_to(void* const ptr, int8_t s8)
{
	union type_punned_cast_t* const tmp = ptr;
	tmp->s8 = s8;
	return;
}

/** Ѵ
 *@param[in,out]	ptr					ѴγǼ
 *@param			s16					Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline void type_punned_cast_s16_to(void* const ptr, int16_t s16)
{
	union type_punned_cast_t* const tmp = ptr;
	tmp->s16 = s16;
	return;
}

/** Ѵ
 *@param[in,out]	ptr					ѴγǼ
 *@param			s32					Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline void type_punned_cast_s32_to(void* const ptr, int32_t s32)
{
	union type_punned_cast_t* const tmp = ptr;
	tmp->s32 = s32;
	return;
}

/** Ѵ
 *@param[in,out]	ptr					ѴγǼ
 *@param			s64					Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline void type_punned_cast_s64_to(void* const ptr, int64_t s64)
{
	union type_punned_cast_t* const tmp = ptr;
	tmp->s64 = s64;
	return;
}


/** Ѵ
 *@param[in]		ptr					Ѵ
 *@return								Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline uint8_t type_punned_cast_to_u8(const void* const ptr)
{
	const union type_punned_cast_t* const tmp = ptr;
	return tmp->u8;
}

/** Ѵ
 *@param[in]		ptr					Ѵ
 *@return								Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline uint16_t type_punned_cast_to_u16(const void* const ptr)
{
	const union type_punned_cast_t* const tmp = ptr;
	return tmp->u16;
}

/** Ѵ
 *@param[in]		ptr					Ѵ
 *@return								Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline uint32_t type_punned_cast_to_u32(const void* const ptr)
{
	const union type_punned_cast_t* const tmp = ptr;
	return tmp->u32;
}

/** Ѵ
 *@param[in]		ptr					Ѵ
 *@return								Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline uint64_t type_punned_cast_to_u64(const void* const ptr)
{
	const union type_punned_cast_t* const tmp = ptr;
	return tmp->u64;
}

/** Ѵ
 *@param[in]		ptr					Ѵ
 *@return								Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline int8_t type_punned_cast_to_s8(const void* const ptr)
{
	const union type_punned_cast_t* const tmp = ptr;
	return tmp->s8;
}

/** Ѵ
 *@param[in]		ptr					Ѵ
 *@return								Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline int16_t type_punned_cast_to_s16(const void* const ptr)
{
	const union type_punned_cast_t* const tmp = ptr;
	return tmp->s16;
}

/** Ѵ
 *@param[in]		ptr					Ѵ
 *@return								Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline int32_t type_punned_cast_to_s32(const void* const ptr)
{
	const union type_punned_cast_t* const tmp = ptr;
	return tmp->s32;
}

/** Ѵ
 *@param[in]		ptr					Ѵ
 *@return								Ѵ
 *
 *@author	G-HAL
 *@date		Sun,02 Oct,2016
 */
static inline int64_t type_punned_cast_to_s64(const void* const ptr)
{
	const union type_punned_cast_t* const tmp = ptr;
	return tmp->s64;
}



#ifdef __cplusplus
}
#endif

#endif /* type_punned_cast_h_included_ */
/* [ End of File ] */
